#!/usr/bin/env python
from __future__ import generators
# import findrox; findrox.version(1,9,12)
# it uses a patched version of ROX-Lib2 in rox/
import rox
from rox import g, tasks
import os
import gc
import gobject

#global minsize ### a Identifier for miniatur icons
#global minsize

class Connections:
	def __init__(self):
		self.__connections = []
	def connect(self, source, signal, callback):
		con = source.connect(signal, callback)
		self.__connections.append({'source' : source, 'signal' : signal, 'con' : con})
	def disconnect_all(self, dummy = None):
		for con in self.__connections: con['source'].disconnect(con['con'])
		self.__connections = []
	def disconnect(self, source, signal):
		i = 0
		while i < len(self.__connections):
			con = self.__connections[i]
			if con['source'] == source and con['signal'] == signal:
				con['source'].disconnect(con['con'])
				self.__connections.pop(i)
			else: i += 1

class Win_ProgBar(g.Window):
	def __init__(self, Label = None):
		g.Window.__init__(self)
		self.set_decorated(g.FALSE)
		self.prog = g.ProgressBar()
		self.prog.set_size_request(100,6)
		self.prog.set_pulse_step(0.2)
		if Label: self.prog.set_text(Label)
		self.add(self.prog)
		x, y, mods = g.gdk.get_default_root_window().get_pointer()
		if x - 50 > 0: x = x - 50
		else: x = 0
		if y - 10 > 0: y = y - 10
		else: y = y + 10
		self.move(x, y)
		self.show_all()
	def start_pulse(self): tasks.Task(self.pulse_loop())
	def pulse_loop(self): 
		while(self.prog):
			self.prog.pulse()
			yield tasks.TimeoutBlocker(0.1)
	def rem(self): self.prog = None; self.destroy()
	

class EventImage(g.EventBox):
	def __init__(self, Options, StockOrTicker = g.STOCK_MISSING_IMAGE, \
				press_event = None, size = g.ICON_SIZE_SMALL_TOOLBAR, tooltip = None):
		self.__Options = Options
		self.size = size
		g.EventBox.__init__(self)
		self.image = g.Image()
		self.add(self.image)
		
		self.set_events(g.gdk.ENTER_NOTIFY_MASK | g.gdk.LEAVE_NOTIFY_MASK)
		if press_event <> None:
			self.connect('button_press_event', self.button_pressed)
			self.connect('button_press_event', press_event)
			self.connect('button_release_event', self.button_released)
		self.connect("enter-notify-event", self.enter_widget)
		self.connect("leave-notify-event", self.leave_widget)

		self.Cons = Connections()
		self.connect("destroy", self.disconnect_all)
		self.__iconset = None
		
		self.set(StockOrTicker)
		
		self.image.show()
		self.show()
		
		if tooltip: self.set_tip(tooltip)
		
		self.__active = 1

		

	def disconnect_all(self, dummy=None): self.Cons.disconnect_all()

	### Changing the States of the Image
	def enter_widget(self, widget=None, event=None): self.image.set_state(g.STATE_PRELIGHT)
	def leave_widget(self, widget=None, event = None): self.image.set_state(g.STATE_NORMAL)
	def set_active(self, active): self.image.set_sensitive(active)
	def button_pressed(self, widget, event): self.image.set_state(g.STATE_SELECTED); tasks.Task(self.blink())
	def button_released(self, widget, event): self.image.set_state(g.STATE_NORMAL)
	def blink(self):
		for i in range(0, 6):
			self.image.set_state(g.STATE_PRELIGHT); yield tasks.TimeoutBlocker(0.02)
			self.image.set_state(g.STATE_NORMAL); yield tasks.TimeoutBlocker(0.02)
	
	### The set Method refers to the set_ method - this makes it easyer to override it (see output.py)
	def set(self, StockOrTicker): self.set_(StockOrTicker)
	def set_(self, StockOrTicker):
		self.Cons.disconnect(self.__iconset, "icon_changed")
		if StockOrTicker.find('gtk-') == 0:
			self.image.set_from_icon_set(self.__Options.stock_iconsets.get(StockOrTicker), self.size)
		else: 
			self.__iconset = self.__Options.sources.get_iconset_by_ticker(StockOrTicker)
			if self.__iconset:
				self.Cons.connect(self.__iconset, "icon_changed", self.refresh) # To keep Icon up to date
				self.Cons.connect(self.__iconset, "icon_at_url_not_retrieved", self.no_icon) # To react if no icon 
				self.refresh()
			else: self.set_(g.STOCK_DIALOG_ERROR) ### maybe Source was killed in meantime
	def refresh(self, dummy = None): self.image.set_from_icon_set(self.__iconset.get(), self.size)
	def no_icon(self, dummy = None): self.image.set_from_icon_set(self.__Options.stock_iconsets.get(g.STOCK_MISSING_IMAGE), self.size)
	def set_tip(self, Text):
		if not(Text): return
		self.tooltip = g.Tooltips(); self.tooltip.enable()
		self.tooltip.set_tip(self, Text)
	def rem_tip(self):
		self.tooltip.disable()

import re
def unxml(string):
	""" removes tags and entities from a given string and returns
	new string """
	lt=re.compile('&lt;')
	gt=re.compile('&gt;')
	amp=re.compile('&amp;')
	auml=re.compile('&auml;|&#228;')
	Auml=re.compile('&Auml;')
	ouml=re.compile('&ouml;|&#246;')
	Ouml=re.compile('&Ouml;')
	uuml=re.compile('&uuml;|&#252;')
	Uuml=re.compile('&Uuml;')
	szlig=re.compile('&szlig;|&#223;')
	quot=re.compile('&quot;')
	apos=re.compile('&apos;')
	#tag=re.compile('<.*?>')
	(string,null)=amp.subn("&",string,0)
	(string,null)=auml.subn(u'\xE4',string,0)
	(string,null)=Auml.subn(u'\xC4',string,0)
	(string,null)=ouml.subn(u'\xF6',string,0)
	(string,null)=Ouml.subn(u'\xD5',string,0)
	(string,null)=Uuml.subn(u'\xDC',string,0)
	(string,null)=szlig.subn(u'\xDF',string,0)
	(string,null)=uuml.subn(u'\xFC',string,0)
	(string,null)=quot.subn('"',string,0)
	(string,null)=apos.subn("'",string,0)
	(string,null)=lt.subn("<",string,0)
	(string,null)=gt.subn(">",string,0)
	#(string,null)=tag.subn("",string,0)
	return string

import time
def conv_iso_time(date):
	if date[-5] == "+" or date[-5] == "-": date = date[:-5]
	if date[-6] == "+" or date[-6] == "-": date = date[:-6]
	return time.mktime(time.strptime(date, "%Y-%m-%dT%H:%M:%S"))

def getdata(channel,name):
	c_name_s=channel.getElementsByTagName(name)
	c_name = ""
	if c_name_s:
		for t in c_name_s[0].childNodes:
			if t: c_name+=t.data
	return c_name

def getdata_wo_ret(channel,name): ### get data as String without return
	c_name_s=channel.getElementsByTagName(name)
	c_name = ""
	if c_name_s:
		for t in c_name_s[0].childNodes:
			if t: c_name+=t.data
	pos = c_name.find('\n')
	while pos <> -1:
		if pos <> 0: tmp = c_name[:pos] + " "
		else: tmp = ""
		tmp += c_name[pos+1:]
		c_name = tmp
		pos = c_name.find('\n')
	return c_name

def getintdata(channel,name):
	c_name_s=channel.getElementsByTagName(name)
	c_name = ""
	if c_name_s:
		for t in c_name_s[0].childNodes:
			if t: c_name+=t.data
	try: return int(c_name)
	except: return 0
def getfloatdata(channel,name):
	c_name_s=channel.getElementsByTagName(name)
	c_name = ""
	if c_name_s:
		for t in c_name_s[0].childNodes:
			if t: c_name+=t.data
	try: return float(c_name)
	except: return 0

def gettimedata(channel,name):
	c_name_s=channel.getElementsByTagName(name)
	c_name = ""
	if c_name_s:
		for t in c_name_s[0].childNodes:
			if t: c_name+=t.data
	if c_name == "": return 0
	else: return float(conv_iso_time(c_name))

def get_prelight_colors(Options):
	back_obj = g.gdk.color_parse(Options.back_color.value)
	text_obj = g.gdk.color_parse(Options.text_color.value)
	if Options.transparent.int_value == 1:
		if text_obj.red + text_obj.green + text_obj.blue > 65535/2:
			text_obj = change_color(text_obj, 0.3)
		else: text_obj = change_color(text_obj, -0.3)
	if text_obj.red + text_obj.green + text_obj.blue \
		> back_obj.red + back_obj.green + back_obj.blue:
		# Text heller als Hintergrund
		text_obj = change_color(text_obj, 0.3)
		back_obj = change_color(back_obj, -0.3)
	else:
		text_obj = change_color(text_obj, -0.3)
		back_obj = change_color(back_obj, 0.3)
	return text_obj, back_obj

def set_color_part(color_part, value):
	tmp = int(color_part * (1 + value))
	if tmp > 65535: return 65535
	if tmp < 0: return 0 # this shouldnt happen ;)
	return tmp

def change_color(obj, value):	
	obj.red = set_color_part(obj.red, value)
	obj.green = set_color_part(obj.green, value)
	obj.blue = set_color_part(obj.blue, value)
	obj.pixel = 0
	return obj


def back_color_prelight(Options):
	text_obj, back_obj = get_prelight_colors(Options)
	return back_obj

def text_color_prelight(Options):
	text_obj, back_obj = get_prelight_colors(Options)
	return text_obj

def tmp_name(cache, url):
	pos = url.find("http://")
	if pos <> -1: url = url[pos+7:]
	while 1:
		pos = url.find("/")
		if pos <> -1:
			url = url[:pos] + "_" + url[pos+1:]
		else: break
	return cache + "/" + url

def makeshorter(text, length):
	new_text = text[:length]
	last_space = new_text[10:].find(" ")
	if last_space <> 0: new_text = new_text[:len(new_text)-10+last_space]
	return new_text + "..."


def urlretrieve(url, file):
	###### special approach to stop blocking in urllib
	if os.path.isfile(file): os.remove(file)
	exec_file = os.path.join(rox.app_dir, 'gethttp.py')
	command = ['python', exec_file, url, file]
	import task
	t = task.Task(command); t.Run()
	return t
	####### feed copyd without blocking

class WaitContainer:
	def __init__(self, time_to_wait, function_to_call):
		self.time_to_wait = time_to_wait
		self.function_to_call = function_to_call
		self.value = None
		self.timeout = None
	def set_time(self, time):
		self.time_to_wait = time
		if self.timeout:
			g.timeout_remove(self.timeout)
			self.timeout = g.timeout_add(long(self.time_to_wait * 1000), self.fire)
	def set_value(self, value = None):
		self.value = value
		#print "set_value"
		if self.timeout: g.timeout_remove(self.timeout)
		if self.time_to_wait <> 0:
			self.timeout = g.timeout_add(long(self.time_to_wait * 1000), self.fire)
		else: self.timeout = None; self.fire()
	def fire(self):
		#print "fired"
		self.fire_task = None
		self.function_to_call(self.value)
		self.value = None

